// jshint maxerr: 500
jQuery(document).ready(function($) {
	let urlParams = new URLSearchParams(window.location.search); // get url params
	
	if ( urlParams.has('tab') && sessionStorage.getItem('success_notice') ) { // check for tab param and session storage to show notice
		switch( urlParams.get('tab') ){ // look for tab value to decide notice message
			case 'http_s':
				createAdminNotice(sessionStorage.getItem('http_conversion') ? search_replace_plugin.httpToHttpsSuccessText : search_replace_plugin.httpsToHttpSuccessText, 'success');
				sessionStorage.removeItem('http_conversion');
				break;
			case 'www_non':
				createAdminNotice(sessionStorage.getItem('www_conversion') ? search_replace_plugin.addWWWSuccessText : search_replace_plugin.removeWWWSuccessText, 'success');
				sessionStorage.removeItem('www_conversion');
				break;
			case 'db_backup':
				createAdminNotice(search_replace_plugin.backupSuccessText, 'success');
				break;
			case 'sql_import':
				createAdminNotice(search_replace_plugin.sqlImportSuccessText, 'success');
				break;
			case 'table_prefix':
				createAdminNotice(search_replace_plugin.changeTablePrefixSuccessText, 'success');
				break;
			default: break;
		}
		sessionStorage.removeItem('success_notice');
	}
	
	var searchedText = $('input[name="searchedText"]');
	var searchButton = $('input[name="searchButton"]');
	var result_content = $('div[name="search_results_content"]');
	var caseSensitive = $('input[name="caseSensitive"]');
	var searchSpinner = $('#search_replace .spinner');
	
	searchedText.keypress(function (e) { // when enter is pressed action fired
		if ((e.keyCode || e.which) === 13) {
		    searchButton.click();
		}
	});
	
	searchButton.on('click', function() {
		var text;
		caseSensitive.attr('checked') ? text = searchedText.val() : text = searchedText.val().toLowerCase(); // check case sensitive if it is not make all text lower case

		searchSpinner.addClass('is-active'); // activate loading spinner
		
		if (text && text.length > 2) { // if text exists and length of it longer than 2
			
			result_content.empty(); // empty if old results

			$.post(search_replace_plugin.ajaxURL, {
					action: 'search', // fire search action
					text: text, // searched text
					isCaseSensitive: caseSensitive.attr('checked') ? true : false, // case sensitive boolean
				},
				function(resultObj) {
					if (resultObj.length !== 0) { // if result is not empty
						removeAdminNotice(); // clear admin notices
						
						var output =	'<table class="wp-list-table widefat fixed striped posts" id="search_replace_table">' +
										'	<thead>' +
										'		<tr>' + // sub links
										'			<td id="cb" class="manage-column column-cb check-column">' +
										'				<label class="screen-reader-text" for="search_selectAll">' + search_replace_plugin.selectAllText + '</label>' +
										'				<input name="search_selectAll" type="checkbox" checked>' +
										'			</td>' +
										'			<th scope="col" class="manage-column">' + search_replace_plugin.contentText + '</th>' +
										'			<th scope="col" class="manage-column">' + search_replace_plugin.tableText + '</th>' +
										'			<th scope="col" class="manage-column">' + search_replace_plugin.columnText + '</th>' +
										'		</tr>' +
										'	</thead>' +
										'	<tbody id="search_replace_rows">'; // found rows starts in here
						var subsubsub = '';
						var count, totalCount = 0;
						
						for (var tableName in resultObj ) {
							count = 0;
							for (var columnName in resultObj[tableName]) {
								if (columnName !== "primary_key") {
									count += Object.keys(resultObj[tableName][columnName].rows).length; // count number of rows in this table

									for (var key in resultObj[tableName][columnName].rows) {

										output +=	'<tr name="' + tableName + '">' +
													'	<th scope="row" class="check-column">' + 
													'		<input id="' + tableName + '-' + columnName + '-' + resultObj[tableName].primary_key + '-' + key + '-' + resultObj[tableName][columnName].rows[key].isSerialized + '" name="' + tableName + '" type="checkbox" checked>' +
													'	</th>';

										var rowSlice = JSON.stringify(escapeHtml(resultObj[tableName][columnName].rows[key].text)); // escape html inside rows
										
										if (rowSlice.length > 50) { // if row length longer than 50 chars do slice for that row to show on page
											var index;
											caseSensitive.attr('checked') ? index = rowSlice.indexOf(text) : index = rowSlice.search(new RegExp(text, "i")); // make case sensitive search and slice to show on page

											if(index === -1){ // if not found search with ' ' empty strings contents
												caseSensitive.attr('checked') ? index = rowSlice.indexOf(text.replace(' ', '%20')) : index = rowSlice.search(new RegExp(text.replace(' ', '%20'), "i"));
											}

											if (index !== -1) { // if any match found get a slice of row that includes searched text and show on page
												var start, end;
												(index - 20) > 0 ? start = index - 20 : start = 0;
												(index + text.length + 20) > rowSlice.length  ? end = rowSlice.length :  end = (index + text.length + 20);
												rowSlice = rowSlice.substring(start, end);
											} else { // if no match found until here that means javascript search could not find any occurence with unknown reason so show searched text directly to show on page it exists here
												rowSlice = "{ " + text + " }";
											}
	
										}

										output +=	'	<td>' + rowSlice + '</td>' +
													'	<td>' + tableName + '</td>' +
													'	<td>' + columnName + '</td>' +
													'</tr>';
									}
								}
							}

							subsubsub += '	<li class="' + tableName + '"><label><input type="checkbox" name="' + tableName + '" checked><a name="subsubsub" href="#">' + tableName + '</a> <span class="count">(' + count + ')</span></label> |</li>'; // sub links with number of rows

							totalCount += count; // total rows count
						}

						output +=	'	</tbody>' +
									'	<tfoot>' + // put sub links on table footer
									'		<tr>' +
									'			<td id="cb" class="manage-column column-cb check-column">' +
									'				<label class="screen-reader-text" for="search_selectAll">' + search_replace_plugin.selectAllText + '</label>' +
									'				<input name="search_selectAll" type="checkbox" checked>' +
									'			</td>' +
									'			<th scope="col" class="manage-column">' + search_replace_plugin.contentText + '</th>' +
									'			<th scope="col" class="manage-column">' + search_replace_plugin.tableText + '</th>' +
									'			<th scope="col" class="manage-column">' + search_replace_plugin.columnText + '</th>' +
									'		</tr>' +
									'	</tfoot>' +
									'</table>';

						subsubsub = subsubsub.substring(0, subsubsub.length - 7); // remove " |</li>" that part of last sub link

						subsubsub = '	<ul class="subsubsub">' +
									'		<li class="all"><label><input type="checkbox" name="selectAll" checked><a name="subsubsub" href="#">' + search_replace_plugin.allText + '</a> <span class="count">(' + totalCount + ')</span></label> |</li>' + subsubsub;
						subsubsub += '	</li></ul>'; // sub links list element finishes here

						var replaceDiv =	'<div class="tablenav top search-replace-settings" name="replaceDiv">' +
											'		<input class="all-options sr-input" type="search" name="replaceText" />' +
											'		<input class="button" type="submit" name="replaceButton" value="' + search_replace_plugin.replaceButtonText + '" />' +
											'		<span class="spinner" style="float:none;"></span>' +
											'</div>';

						output = subsubsub + replaceDiv + output + replaceDiv; // concatenate all elements

						result_content.append(output); // put them parent result div

						initResults(); // fire put elements actions
						
						searchSpinner.removeClass('is-active');

					} else { // if result is empty
						createAdminNotice(search_replace_plugin.notFoundTitle, 'info'); // not found admin notice
						searchSpinner.removeClass('is-active'); // remove spinner
					}

				});
		
		} else {
			createAdminNotice(search_replace_plugin.searchResultEmpty, 'info');
			searchSpinner.removeClass('is-active');
		}
	});
	
	// Generic remove current admin notice fuction.
	function removeAdminNotice(){
		var past_div = $('#notice_div');
		past_div.length > 0 ? past_div.remove() : false;
	}
	
	// Generic create admin notice with given message, type(etc. info, error, success) and confirm button or not
	function createAdminNotice(message, type, showConfirmButton = false){
		removeAdminNotice(); // remove first if any notice exists
		
		var div = $('<div></div>').addClass('notice notice-'+type+' is-dismissible').attr('id', 'notice_div');
		var p = $('<p></p>').html(message);
		
		div.append(p);
		
		var button = $('<button></button>').attr('type', 'button').addClass('notice-dismiss');
		var buttonSpan = $('<span></span>').addClass('screen-reader-text').html('Dismiss this notice');
		
		button.append(buttonSpan);
		div.append(button);
		
		if( showConfirmButton ){ // if confirm button true create and show
			var okButton = $('<button></button>').attr('type', 'submit').html('OK').addClass('button-secondary');
			var pButton = $('<p></p>').append(okButton);
			div.append(pButton);
		}
		
		var navTabs = $('nav[class="nav-tab-wrapper"]');
		
		div.insertAfter(navTabs);
		
		button.on( 'click', function () {
	        removeAdminNotice();
	    });
	    
		return (showConfirmButton ? okButton :  true); // if confirm button true return that to put click action when this function called
	}
	
	// Function for after page load added elements actions.
	function initResults() {
		var replaceButton = $('input[name="replaceButton"]');
		var replaceSpinners = $('div[name="replaceDiv"] .spinner');
	
		var selectAll = $('input[name="selectAll"]');
		var searchSelectAlls = $('input[name="search_selectAll"]');
		var subsubsubCheckboxes = $('.subsubsub input:checkbox:not([name="selectAll"])');
		var rowCheckboxes = $('#search_replace_rows input:checkbox');
		
		rowCheckboxes.change(function() {
			var checkedRowCount = $('#search_replace_rows input:checkbox:checked').length; // number of checked rows from all rows
			var uncheckedRowCount = $('#search_replace_rows input:checkbox:not(:checked)').length; // number of not checked rows from all rows
			
			var checkedTableRowCount = $('#search_replace_rows input[name="' + this.name + '"]:checked').length; // number of checked rows from same table's rows with current row
			var uncheckedTableRowCount = $('#search_replace_rows input[name="' + this.name + '"]:not(:checked)').length; // number of not checked rows from same table's rows with current row
			
			var tableCheckbox = $('.subsubsub input[name="' + this.name + '"]'); // get current row table sub link checkbox
			
			if (!this.checked) { // if current row is unchecked
				if(checkedTableRowCount < 1){ // if no checked checkbox exist from same table
					tableCheckbox.removeClass('indeterminate');
					tableCheckbox.removeAttr('checked');

					if(checkedRowCount < 1){ // if no checked checkbox exist from all rows
						selectAll.removeClass('indeterminate');
						selectAll.removeAttr('checked');

						searchSelectAlls.removeClass('indeterminate');
						searchSelectAlls.removeAttr('checked');
					} else { // if any checked checkbox exist from all rows
						searchSelectAlls.removeAttr('checked');
						searchSelectAlls.addClass('indeterminate');

						selectAll.removeAttr('checked');
						selectAll.addClass('indeterminate');
					}

				} else { // if any checked checkbox exist from same table
					searchSelectAlls.removeAttr('checked');
					searchSelectAlls.addClass('indeterminate');

					tableCheckbox.removeAttr('checked');
					tableCheckbox.addClass('indeterminate');

					selectAll.removeAttr('checked');
					selectAll.addClass('indeterminate');
				}

			} else { // if current row is checked
				if(uncheckedTableRowCount < 1){ // if no unchecked checkbox exist from same table
					tableCheckbox.removeClass('indeterminate');
					tableCheckbox.attr('checked', 'checked');

					if(uncheckedRowCount < 1){ // if no unchecked checkbox exist from all rows
						searchSelectAlls.removeClass('indeterminate');
						searchSelectAlls.attr('checked', 'checked');

						selectAll.removeClass('indeterminate');
						selectAll.attr('checked', 'checked');
					} else { // if any unchecked checkbox exist from all rows
						searchSelectAlls.removeAttr('checked');
						searchSelectAlls.addClass('indeterminate');

						selectAll.removeAttr('checked');
						selectAll.addClass('indeterminate');
					}
				} else { // if any unchecked checkbox exist from same table
					tableCheckbox.removeAttr('checked');
					tableCheckbox.addClass('indeterminate');

					searchSelectAlls.removeAttr('checked');
					searchSelectAlls.addClass('indeterminate');

					selectAll.removeAttr('checked');
					selectAll.addClass('indeterminate');
				}
			}
		});
		
		subsubsubCheckboxes.change(function() {
			var checkedTableCount = $('.subsubsub input:checkbox:not([name="selectAll"]):checked').length; // checked sub links except select all 
			var uncheckedTableCount = $('.subsubsub input:checkbox:not([name="selectAll"]):not(:checked)').length; // not checked sub links except select all 
			
			if (!this.checked) { // if a sub link (table checkbox) unchecked
				subsubsubCheckboxes.removeClass('indeterminate');
				if(checkedTableCount < 1){ // if no checked sub link left
					searchSelectAlls.removeClass('indeterminate');
					searchSelectAlls.removeAttr('checked');
					
					selectAll.removeClass('indeterminate');
					selectAll.removeAttr('checked');
				} else { // if any checked sub link exist
					searchSelectAlls.removeAttr('checked');
					searchSelectAlls.addClass('indeterminate');
					
					selectAll.removeAttr('checked');
					selectAll.addClass('indeterminate');
				}
				
				$('#search_replace_rows input[name="' + this.name + '"]:checked').removeAttr('checked'); // uncheck all checked rows that is from current table
			} else { // if a sub link (table checkbox) checked
				$('#search_replace_rows input[name="' + this.name + '"]:not(:checked)').attr('checked', 'checked'); // check all unchecked rows that is from current table
				var uncheckedRowCount = $('#search_replace_rows input:checkbox:not(:checked)').length; // all not checked rows 
				
				if(uncheckedRowCount < 1){ // if no not-checked row left
					searchSelectAlls.removeClass('indeterminate');
					searchSelectAlls.attr('checked', 'checked');
					
					selectAll.removeClass('indeterminate');
					selectAll.attr('checked', 'checked');
				} else { // if any not-checked row exist
					selectAll.addClass('indeterminate');
					searchSelectAlls.addClass('indeterminate');
				}
			}
		});
		
		searchSelectAlls.change(function() {
			selectAll.removeClass('indeterminate');
			subsubsubCheckboxes.removeClass('indeterminate');
			
			if( this.checked ) { // if select all checkbox inside table checked
				selectAll.attr('checked', 'checked');
				subsubsubCheckboxes.attr('checked', 'checked');
				rowCheckboxes.attr('checked', 'checked');
			} else { // if select all checkbox inside table unchecked
				searchSelectAlls.removeClass('indeterminate');
				selectAll.removeAttr('checked');
				subsubsubCheckboxes.removeAttr('checked');
				rowCheckboxes.removeAttr('checked');
			}
		});
		
		selectAll.change(function() {
			searchSelectAlls.removeClass('indeterminate');
			subsubsubCheckboxes.removeClass('indeterminate');
			
			if( this.checked ) { // if select all sub link checked
				searchSelectAlls.attr('checked', 'checked');
				subsubsubCheckboxes.attr('checked', 'checked');
				rowCheckboxes.attr('checked', 'checked');
			} else { // if select all sub link unchecked
				selectAll.removeClass('indeterminate');
				searchSelectAlls.removeAttr('checked');
				subsubsubCheckboxes.removeAttr('checked');
				rowCheckboxes.removeAttr('checked');
			}
		});
		
		replaceButton.on('click', function() {
			var replaceText = $(this).prev(); // replace text input
			var newText = replaceText.val();
			var oldText = searchedText.val();
			var updateRows = [];
			var rowCheckboxes = $('#search_replace_rows input:checkbox:checked'); // all checked rows
			
			$.each(rowCheckboxes, function(index, value) { // split row info inside id to fill update rows array
				if (value.checked) {
					updateRows.push(value.id);
				}
			});
			
			if( ! newText ){ // if replace text does not exist
				createAdminNotice(search_replace_plugin.replaceTextEmpty, 'info');
				return;
			}
			
			if( updateRows.length === 0 ){ // if no row checked
				createAdminNotice(search_replace_plugin.replaceNoRowSelected, 'info');
				return;
			}
			
			var okButton = createAdminNotice(search_replace_plugin.confirmText, 'warning', true); // create a notice with confirm needed
			okButton.on('click', function(){ // returned confirm button click action
				removeAdminNotice();
				replaceSpinners.addClass('is-active');

				$.post(search_replace_plugin.ajaxURL, {
					action: 'replace',
					oldText: oldText,
					newText: newText,
					isCaseSensitive: caseSensitive.attr('checked') ? true : false,
					updateRows: updateRows
				}).done(function(data) {
					replaceSpinners.removeClass('is-active');
					var numOfReturnedQuery = parseInt(data.split('/')[1]); // split response message (ex: 199/199)
					result_content.empty();
					if( updateRows.length == numOfReturnedQuery ){ // if sent and returned number of rows are equal
						createAdminNotice(search_replace_plugin.replaceSuccessText, 'success');
					} else { // if sent and returned number of rows are not equal
						createAdminNotice(data + search_replace_plugin.replaceLimitInfo + updateRows.length + '. ' + search_replace_plugin.replaceLimitInfo2, 'info');
					}
				}).fail(function(data){ // any case of error
					replaceSpinners.removeClass('is-active');
					result_content.empty();
					createAdminNotice(data.responseText, 'error');
				});
			});

		});
		
		var subsubsubLinks = $('a[name="subsubsub"]');
		
		subsubsubLinks.on('click', function (){ // sub links show-hide actions
			if(this.text === 'All'){
				$('#search_replace_rows tr').show();
			} else {
				var hideRows = $('#search_replace_rows tr:not([name="'+this.text+'"])');
				var showRows = $('#search_replace_rows tr[name="'+this.text+'"]');
				
				showRows.show();
				hideRows.hide();
			}
		});
		
	}

	function escapeHtml(unsafe) {
		return unsafe
			.replace(/&/g, "&amp;")
			.replace(/</g, "&lt;")
			.replace(/>/g, "&gt;")
			.replace(/"/g, "&quot;")
			.replace(/'/g, "&#039;");
	}

	var applyHttpsOptions = $('input[name="applyHttpsOptions"]');
	var httpToHttps = $('input[data-name="httpToHttps"]');
	var httpSpinner = $('#http_s .spinner');
	
	applyHttpsOptions.on('click', function() {
		if( $('input[data-name="httpsToHttp"]').is(':checked') || httpToHttps.is(':checked') ){ // if any radio button checked
			var okButton = createAdminNotice(search_replace_plugin.confirmText, 'warning', true);
			okButton.on('click', function(){
				removeAdminNotice();
				httpSpinner.addClass('is-active');
				
				$.post(search_replace_plugin.ajaxURL, {
					action: 'httpHttps',
					add_s: httpToHttps.is(':checked'),
				},
				function(data) {
					httpSpinner.removeClass('is-active');
					if( parseInt(data) === 1 ){ // if response true
						sessionStorage.setItem('http_conversion', httpToHttps.is(':checked')); // set which action done
						sessionStorage.setItem('success_notice', true);
						location.reload(true);
					} else { // if error message returned
						createAdminNotice(data, 'error');
					}
				});
				
			});
		} else { // if no radio button checked
			createAdminNotice(search_replace_plugin.emptyTitle, 'info');
		}
	});
	
	var applyWWWOptions = $('input[name="www_non_www"]');
	var addWWW = $('input[data-name="addWWW"]');
	var wwwSpinner = $('#www_non .spinner');

	applyWWWOptions.on('click', function() {
		if( $('input[data-name="removeWWW"]').is(':checked') || addWWW.is(':checked') ){ // if any radio button checked
			var okButton = createAdminNotice(search_replace_plugin.confirmText, 'warning', true);
			okButton.on('click', function(){
				removeAdminNotice();
				wwwSpinner.addClass('is-active');
				
				$.post(search_replace_plugin.ajaxURL, {
					action: 'wwwNoNwww',
					add_www: addWWW.is(':checked'),
				},
				function(data) {
					wwwSpinner.removeClass('is-active');
					if( parseInt(data) === 1 ){ // if response true
						sessionStorage.setItem('www_conversion', addWWW.is(':checked')); // set which action done
						sessionStorage.setItem('success_notice', true);
						location.reload(true);
					} else { // if error message returned
						createAdminNotice(data, 'error');
					}
				});
			});
		} else { // if no radio button checked
			createAdminNotice(search_replace_plugin.emptyTitle, 'info');
		}
	});

	var backup = $('input[name="backup"]');
	var backupSpinner = $('#db_backup .spinner');

	backup.on('click', function() {
		backupSpinner.addClass('is-active');
		$.post(search_replace_plugin.ajaxURL, {
				action: 'backup'
			},
			function(data) {
				backupSpinner.removeClass('is-active');
				sessionStorage.setItem('success_notice', true);
				location.reload(true);
			});
	});

	var sqlImport = $('input[name="sqlImport"]');
	var sqlFileUpload = $('input[name="sqlFileUpload"]');
	var sqlImportSpinner = $('#sql_import .spinner');
	
	sqlFileUpload.on('change', function(){ // enable import button when a file imported
		if (sqlFileUpload.prop('files')[0]) {
			sqlImport.removeAttr("disabled");
		}
	});
	
	sqlImport.on('click', function() {
		if (sqlFileUpload.prop('files')[0]) { // if any file imported
			if (sqlFileUpload.prop('files')[0].name.split('.').pop() !== "sql") { // if imported file extension is not .sql
				createAdminNotice(search_replace_plugin.noneSQLFileImportError, 'error');
				sqlFileUpload.val('');
			} else { // if imported file extension is .sql
				var formdata = new FormData();

				formdata.append('action', 'sqlImport');
				formdata.append('sqlFile', sqlFileUpload.prop('files')[0]); // append file to form data

				var okButton = createAdminNotice(search_replace_plugin.confirmText, 'warning', true);
				okButton.on('click', function(){
					removeAdminNotice();
					sqlImportSpinner.addClass('is-active');
					
					$.ajax({
						type: "post",
						url: search_replace_plugin.ajaxURL,
						data: formdata,
						processData: false,
						contentType: false,
						success: function(data) {
							if (data.length < 1 || data[0] === '') { // successful response
								sessionStorage.setItem('success_notice', true);
								location.reload(true);
							} else { // response with an error message
								createAdminNotice(data, 'error');
							}
							sqlImportSpinner.removeClass('is-active');
							sqlFileUpload.val('');
						}
					});
				});
			}
		}
	});

	$('input[name="radio"]').on('change', function() {
		$('input[name="radio"]').not(this).prop('checked', false);
	});

	var customSqlImport = $('input[name="customSqlImport"]');
	var customSqlResults = $('div[name="customSqlResults"]');
	var customSqlSpinner = $('#custom_sql_query .spinner');

	customSqlImport.on('click', function() {
		var customSqlText = $('textarea[name="customSqlText"]');

		if (customSqlText.val()) {
			var okButton = createAdminNotice(search_replace_plugin.confirmText, 'warning', true);
			okButton.on('click', function(){
				removeAdminNotice();
				customSqlSpinner.addClass('is-active');
				
				$.post(search_replace_plugin.ajaxURL, {
					action: 'customSqlImport',
					sql: customSqlText.val()
				},
				function(data) {
					if (!data.errors) { // successfull
						createAdminNotice(search_replace_plugin.customSqlImportSuccessText, 'success');
						customSqlResults.empty();
						if (data.aff) {
							customSqlResults.append('<strong class="row m-3">Affected number of rows: ' + data.aff + '</strong>'); // show affected rows info
						}
						var out = '';
						for (var row in data.rows) {
							out += '<div class="row m-3">' + row + ' => ';
							for (var item in data.rows[row]) {
								out += escapeHtml(data.rows[row][item]) + ' ';
							}
							out += '</div>';
						}
						customSqlResults.append(out); // show all rows info
					} else { // fail
						createAdminNotice(search_replace_plugin.errorText, 'error');
						customSqlResults.empty();
						customSqlResults.append(data.errors);
					}
					customSqlSpinner.removeClass('is-active');
				});
			});
		}
	});

	var changeTablePrefix = $('input[name="changeTablePrefix"]');
	var changeTablePrefixSpinner = $('#custom_sql_query .spinner');

	$('input[name="tablePrefix"]').keypress(function (e) { // when enter is pressed action fired
		if ((e.keyCode || e.which) === 13) {
		    changeTablePrefix.click();
		}
	});
	
	changeTablePrefix.on('click', function() {
		var tablePrefix = $('input[name="tablePrefix"]');
		var format = /[ !@#$%^&*()_+\-=\[\]{};':"\\|,.<>\/?]/; // format for only letters exclude special chars
		
		if (tablePrefix.val()) { // if a text exist
			if (format.test(tablePrefix.val())) { // if input not matched with format
				createAdminNotice(search_replace_plugin.formatErrorText, 'error');
			} else { // if input not matched with format
				var okButton = createAdminNotice(search_replace_plugin.confirmText, 'warning', true);
				okButton.on('click', function(){
					removeAdminNotice();
					changeTablePrefixSpinner.addClass('is-active');
					
					$.post(search_replace_plugin.ajaxURL, {
						action: 'tablePrefix',
						prefix: tablePrefix.val()
					},
					function(data) {
						if (data) { // if any error message exist
							createAdminNotice(data, 'error');
						} else { // successful
							sessionStorage.setItem('success_notice', true);
							location.reload(true);
						}
						changeTablePrefixSpinner.removeClass('is-active');
					});
				});
			}
		}
	});

});