<?php

defined( 'ABSPATH' ) or die;

if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class Weptile_Backup_List extends WP_List_Table{
	
	/**
	 * Count array for subsubsub links.
	 * Ex: array( 'all' => 40, import' => 15, 'backup' => 25 ) 
	 **/
	protected $counts;
	
	protected $per_page;
	
	protected $upload_dir;
	
	public function __construct( ) {
		$this->upload_dir 	= wp_upload_dir();
		
		$this->counts 		= array( 'all' => 0, 'import' => 0, 'backup' => 0 );
		$this->per_page 	= 10;
		
		parent::__construct( array(
			'plural' 	=> __('Backups', 'weptile-wp-db-search-replace'),
        	'singular' 	=> __('Backup', 'weptile-wp-db-search-replace'),
        	'ajax' 		=> false,
        	'screen' 	=> 'all',
		) );
	}
	
	/**
	 * @description: Saves an instant backup file inside backups folder under weptile-wp-db-search-replace.
	 * @param: $filename = {test-{timestamp}.sql}, $filepath = {.../backups}
	 **/
	public function backup_database($filename, $file_path){
		
		global $wpdb;
		
		$tables = $wpdb->get_results( "SHOW TABLES" ); // get all table
		
		if(! file_exists($file_path)) { // create folder if it does not exist.
			mkdir($file_path, 0777, true); // 0777 default mode for directories.
		}
		
		$file_path = $file_path . '/' . $filename;
	
		$file = fopen($file_path, 'w');
		
		foreach($tables as $table) {
			foreach($table as $tableName) {
	
			    $schema = $wpdb->get_row( $wpdb->prepare( 'SHOW CREATE TABLE ' . $tableName ), ARRAY_A ); // table creation schema with all spesifications.
	
			    fwrite($file, $wpdb->prepare( "DROP TABLE IF EXISTS " . $tableName .';' . PHP_EOL ) ); // write drop table sql command
	
			    fwrite($file, $schema['Create Table'] . ';' . PHP_EOL); // write create table sql command
	
				$rows = $wpdb->get_results( $wpdb->prepare( 'SELECT * FROM ' . $tableName ), ARRAY_A ); // get all rows on table
	
	    		if( $rows ) { // if any row exists
	
			        fwrite($file, 'INSERT INTO ' . $tableName . ' VALUES '); // start to write insert sql command for each row
	
			        $total_rows = count($rows);
	
			        $counter = 1;
	
			        foreach ($rows as $row => $fields) {
	
			            $line = '';
	
			            foreach ($fields as $key => $value) {
			                $value = addslashes($value);
			                $line .= '"' . $value . '",';
			            }
	
			            $line = '(' . rtrim($line, ',') . ')';
	
			            if ($counter != $total_rows) {
			                $line .= ',' . PHP_EOL;
			            }
	
			            fwrite($file, $line);
	
			            $counter++;
			        }
	
			        fwrite($file, '; ' . PHP_EOL); // finish to write insert sql command for each row 
			    }
			}	
		}
	
		fclose($file);
	
	}
	
	/**
	 * @description: Finds given file name on backup folder and restore it.
	 * @param: $file_path (String) => file path to restore.
	 * @return: errors if exist.
	 **/
	public function restore_backup( $restore_file_name ){
		$splitted_file_name = explode('-', $restore_file_name);
		$unix_timestamp = explode( '.', end( $splitted_file_name ) )[0]; // take last element of splitted file name to reach timestamp
		$date = date('Y-m-d', $unix_timestamp); // convert timestamp to date
		
		$file_path = $this->upload_dir['basedir'] . '/weptile-wp-db-search-replace/backups/' . $date . '/' . $restore_file_name;
		
		return $this->restore_file( $file_path );
	}

	/**
	 * @description: Restores file in given path.
	 * @param: $file_path (String) => file path to restore.
	 * @return: errors if exist.
	**/
	public function restore_file( $file_path ){
	
		global $wpdb;
		
		$errors = '';
		
		if( is_readable( $file_path ) ){ // check given file is readable
			$sql = file_get_contents($file_path); // get sql commands
		}
		
		if( $sql ){ // if commands exists
			$errors = $this->custom_sql_import( $sql )['errors']; // send custom sql import file to execute commands
		}
		
		return $errors;
	}
	
	/**
	 * @description: Executes given SQL query/queries.
	 * @param: $sql (String) => given sql query
	 * @return: $info ( 'aff' => {affected rows}, 'rows' => {all rows}, 'errors' => {errors if exist} ).
	 **/
	function custom_sql_import( $sql ){
		
		global $wpdb;
		
		$info = array();
		
		$con = mysqli_connect($wpdb->dbhost, $wpdb->dbuser, $wpdb->dbpassword, $wpdb->dbname); // connect mysqli
	
		if ( mysqli_connect_errno() ){ // if any connection error occurs return error
			return array( 'errors' => __("Failed to connect to MySQL: ", 'weptile-wp-db-search-replace') . mysqli_connect_error() );
		}
		
		$sql = mysqli_real_escape_string($con, $sql); // mysqli escape sql commands
		
		if ( ! mysqli_multi_query($con, $sql) ){ // execute multiple sql commands. if any error occurs fill info array errors key with errors.
			$info['errors'][] =  $con->error;
		} else { // no error occured after multiple query executed
			do {
		        /* store first result set */
		        if ($result = mysqli_store_result($con)) {
		        	$info['aff'][] = mysqli_affected_rows($con); // affected rows
		        	while ($row = mysqli_fetch_row($result)) {
		        		$info['rows'][] = $row; // each row
		        	}
	            	mysqli_free_result($result);
		        } else {
					$info['errors'][] =  $con->error; // get error
		        }
	    	} while (mysqli_more_results($con) && mysqli_next_result($con));
		}
	
		mysqli_close( $con ); // mysqli connection close
		
		return $info;
	}
	
	/**
	 * @description: Look for backup files inside default path. ( /uploads/weptile-wp-db-search-replace/backups/ )
	 * @param: $s => searched value to return file names that contains it
	 * @return: $entries => [ {day} => [ 0 => {file_name}, 1 => '', ... ], ... ]
	 **/
	public function get_backup_files( $s ){
		
		$file_path = $this->upload_dir['basedir'] . '/weptile-wp-db-search-replace/backups'; // saved backups path
		
		if(! file_exists($file_path)) // check path does not exist
	        return array();
		
		$files = scandir($file_path, 1); // get files inside backups file
		
		// remove default files that exist for each file
	    unset( $files[array_search('.', $files, true)] );
	    unset( $files[array_search('..', $files, true)] );
	    
	    $entries = array();
	        
		if( $s !== ''){ // if search text exists
			foreach ( $files as $file ) {
				$entries[$file] = scandir($file_path.'/'.$file, 1); // get all files in path
				
				// remove default files that exist for each file
				unset( $entries[$file][array_search('.', $entries[$file], true)] );
				unset( $entries[$file][array_search('..', $entries[$file], true)] );

				foreach( $entries[$file] as $index => $filename){
					$text = explode('.', $filename)[0]; // get file name before it's extension
					if( preg_match('/'.$s.'/', $text) == 0 ){ // check file name contains search text and if not remove that file from return array
						unset( $entries[$file][$index] );
					}
				}

				$this->counts['all'] += count($entries[$file]); // count all found files

				if($file === 'imported-files'){
					$this->counts['import'] += count($entries[$file]); // count found imported files
				} else {
					$this->counts['backup'] += count($entries[$file]); // count found backup files
				}
		    }
		} else { // if search text does not exist
			foreach ( $files as $file ) {
				$entries[$file] = scandir($file_path.'/'.$file, 1); // get all files in path

				// remove default files that exist for each file
				unset( $entries[$file][array_search('.', $entries[$file], true)] );
				unset( $entries[$file][array_search('..', $entries[$file], true)] );

				$this->counts['all'] += count($entries[$file]); // count all found files

				if($file === 'imported-files'){
					$this->counts['import'] += count($entries[$file]); // count found imported files
				} else {
					$this->counts['backup'] += count($entries[$file]); // count found backup files
				}
		    }
		}
	
		return $entries;
	}

	/**
	 * @description: Delete given filenames with searching all backups.
	 * @param: $delete_files (array) => file names to delete.
	 **/
	public function delete_backup_files( $delete_files ){
		
		$file_path = $this->upload_dir['basedir'] . '/weptile-wp-db-search-replace/backups'; // saved backups path
		
		if(! file_exists($file_path)) // check path does not exist
		    return array();
		
		$files = scandir($file_path, 1); // get files inside backups file
		
		// remove default files that exist for each file
		unset( $files[array_search('.', $files, true)] );
		unset( $files[array_search('..', $files, true)] );
		    
		foreach ( $files as $dir ) {
			$dir_path = $file_path.'/'.$dir; // folder path that exists in backups folder
			if( is_dir( $dir_path ) ){ // check if it is a folder or not
				foreach ( scandir( $dir_path ) as $filename ){ // look for files inside folder
					foreach ( $delete_files as $delete_file ){
						if( $filename === $delete_file ){ // check that file match with any one of the delete file names
							unlink($dir_path.'/'.$filename); // delete matched file
							break;
						}
					}
				}
				if( empty( glob($dir_path.'/*') ) ){ // check that folder is empty or not 
					rmdir($dir_path); // if folder empty delete that folder too
				}
			}
		}
		
	}
	
	/**
	 * @description: Downloads the file that matchs given file name.
	 * @param: $file(String) => file name
	 **/
	public function download_backup( $file ){
		
		$file_path = $this->upload_dir['basedir'] . '/weptile-wp-db-search-replace/backups'; // saved backups path
		
		if(! file_exists($file_path)) // check path does not exist
		    return array();
		
		$files = scandir($file_path, 1); // get files inside backups file
		
		// remove default files that exist for each file
		unset( $files[array_search('.', $files, true)] );
		unset( $files[array_search('..', $files, true)] );
		    
		foreach ( $files as $dir ) {
			$dir_path = $file_path.'/'.$dir; // folder path that exists in backups folder
			if( is_dir( $dir_path ) ){ // check if it is a folder or not
				foreach ( scandir( $dir_path ) as $filename ){ // look for files inside folder
					if( $filename === $file ){ // check file name matchs given file name 
						$found_file_path = $dir_path . '/' . $file;
						if(file_exists($found_file_path)) { // if file exists download
					        header('Content-Description: File Transfer');
							header('Content-Type: application/octet-stream');
					        header('Content-Disposition: attachment; filename="'.basename($found_file_path).'"');
					        header('Expires: 0');
					        header('Cache-Control: must-revalidate');
					        header('Pragma: public');
					        header('Content-Length: ' . filesize($found_file_path));
            				ob_clean();
					        readfile($found_file_path);
					        exit;
					    }
					}
				}
			}
		}
	}

	public function prepare_items() {

     	$customvar = isset( $_REQUEST['customvar'] ) ? $_REQUEST['customvar'] : 'all'; // subsubsub menu slug to show table data
        $s = isset( $_REQUEST['s'] ) ? $_REQUEST['s'] : ''; // searched value
		
        $columns = $this->get_columns();
        $hidden = $this->get_hidden_columns();
        $sortable = $this->get_sortable_columns();
        
        $data = $this->table_data($customvar, $s); // get table data according to current sub menu and searched value
        $perPage = isset( $_REQUEST['per_page'] ) ? intval( $_REQUEST['per_page'] ) : $this->per_page;
        $currentPage = $this->get_pagenum();
        
        $totalItems = $this->counts['all'];

        $this->set_pagination_args( array(
            'total_items' => $totalItems,
            'per_page'    => $perPage
        ) );

        $data = array_slice( $data, ( ( $currentPage - 1 ) * $perPage ), $perPage );

        $this->_column_headers = array( $columns, $hidden, $sortable );
		
		$this->process_bulk_action();
		
        $this->items = $data;
    }
	
	public function get_columns() {
		
        $columns = array(
    		'cb'        => '<input type="checkbox" />',
            'date' 		=> __('Date', 'weptile-wp-db-search-replace'),
            'type'		=> __('Type', 'weptile-wp-db-search-replace'),
            'title'		=> __('Filename', 'weptile-wp-db-search-replace'),
        );
        
        return $columns;
	}
	
	public function get_hidden_columns() {
        return array();
	}
	
	public function get_sortable_columns() {
        return array(
        	'title' => array('title', false),
        	'date' => array('date', false)
        );
	}
	
    public function get_bulk_actions() {

        return array(
            'delete' => __( 'Delete', 'weptile-wp-db-search-replace' ),
        );

    }
    
    public function process_bulk_action() {

        $action = $this->current_action(); // get current action

        switch ( $action ) {

            case 'delete':
				$nonce = esc_attr( $_REQUEST['_wpnonce'] ); // get nonce for security check
            	
				if( isset( $_REQUEST['file'] ) ) {
					if ( ! wp_verify_nonce( $nonce, 'wpdbsr_backup_delete' ) ) { // verify nonce
						wp_die( 'Nope! Security check failed!' );
					} else {
		    			$file = $_GET['file'];
		                $this->delete_backup_files( array( 0 => $file ) ); // delete file
					}
    				wp_redirect( '?page='.esc_attr( $_REQUEST['page'] ).'&tab='.esc_attr( $_REQUEST['tab'] ).'&notice=ds' ); // successfuly refresh page and show admin notice
				} else if ( isset( $_REQUEST['backup'] ) && is_array( $_REQUEST['backup'] ) && ! empty( $_REQUEST['backup'] ) ) {
					if ( ! wp_verify_nonce( $nonce, 'bulk-' . $this->_args['plural'] ) ) { // verify nonce
						wp_die( 'Nope! Security check failed!' );
					} else {
		    			$files = $_GET['backup'];
		    			$this->delete_backup_files( $files ); // delete files
					}
    				wp_redirect( '?page='.esc_attr( $_REQUEST['page'] ).'&tab='.esc_attr( $_REQUEST['tab'] ).'&notice=ds' ); // successfuly refresh page and show admin notice
				} else {
					wp_redirect( '?page='.esc_attr( $_REQUEST['page'] ).'&tab='.esc_attr( $_REQUEST['tab'] ).'&notice=de' ); // error occured refresh page and show admin notice
				}

                break;
            case 'restore':
				$nonce = esc_attr( $_REQUEST['_wpnonce'] ); // get nonce for security check

				if ( ! wp_verify_nonce( $nonce, 'wpdbsr_backup_restore' ) ) { // verify nonce
					wp_die( 'Nope! Security check failed!' );
				} else {
	    			$file = $_GET['file'];
	    			$errors = $this->restore_backup( $file ); // restore file
	                if( empty( $errors ) || empty( $errors[0] ) ){ // if any error exists show error notice
    					wp_redirect( '?page='.esc_attr( $_REQUEST['page'] ).'&tab='.esc_attr( $_REQUEST['tab'] ).'&notice=rs' ); // successfuly refresh page and show admin notice
	                } else {
    					wp_redirect( '?page='.esc_attr( $_REQUEST['page'] ).'&tab='.esc_attr( $_REQUEST['tab'] ).'&notice=re' ); // error occured refresh page and show admin notice
	                }
				}

                break;
            case 'download':
				$nonce = esc_attr( $_REQUEST['_wpnonce'] ); // get nonce for security check
				
				if ( ! wp_verify_nonce( $nonce, 'wpdbsr_backup_download' ) ) { // verify nonce
					wp_die( 'Nope! Security check failed!' );
				} else {
	    			$file = $_GET['file'];
	    			$errors = $this->download_backup( $file ); // download backup file
				}
            	break;
            default:
                return;
                break;
        }

        return;
    }
	
	/**
	 * @description: Overwriten method to put an div element for per page input.
	 * @param: which side top or bottom
	 **/
	public function extra_tablenav( $which ) {
		$per_page = isset( $_REQUEST['per_page'] ) ? esc_attr( $_REQUEST['per_page'] ) : $this->per_page;

		if ( $which == "top" ) { 
			?>
			<div class="alignright actions show-items-count">
				<label for="per_page_input" class="screen-reader-text">Change Row Number Per Page:</label>
				<input type="number" value="<?php echo $per_page; ?>" name="per_page" id="per_page_input">
				<input type="submit" value="Show" class="button" id="per_page-submit" name="">
			</div>
			<?php 
		}
	}

	public function table_data($customvar, $s) {
		// search for s value
		$data = array();
		
		foreach($this->get_backup_files( $s ) as $day => $entry){
			$type = $day === 'imported-files' ? __('Import', 'weptile-wp-db-search-replace') : __('Backup', 'weptile-wp-db-search-replace'); // set type imported or backup according to sub folders name
			
			if( $customvar !== 'all' && (($customvar === 'import') === ($day !== 'imported-files')) ){ // if all sublink not current page and current page not match with folder type pass that folder
				continue;
			}
			
			foreach($entry as $file){
				$splitted_file = explode('-', $file);
				$unix_timestamp = explode( '.', end( $splitted_file ) )[0]; // take last element of splitted file name to reach timestamp
	
				$time = date(get_option('date_format') . ' - ' . get_option('time_format'), $unix_timestamp);  // convert timestamp to current date format
				
				$data[] = array(
		            'title'		=> $file,
		            'type'		=> $type,
		            'date' 		=> $time,
				);
			}
		}
		
		return $data;
	}
	
	public function column_default( $item, $column_name ) {
		
        switch( $column_name ) {
            case 'title':
            case 'type':
            case 'date':
            	return $item[$column_name];
            default:
                return print_r( $item, true ) ;
        }
	}
	
	public function column_date( $item ) {
		
		$delete_nonce = wp_create_nonce( 'wpdbsr_backup_delete' ); // create delete nonce to verify when action fired
		
		$restore_nonce = wp_create_nonce( 'wpdbsr_backup_restore' ); // create backup download nonce to verify when action fired
		
		$download_nonce = wp_create_nonce( 'wpdbsr_backup_download' ); // create backup restore nonce to verify when action fired
		
		$actions = [ // actions and their href
			'restore' => sprintf( '<a href="?page=%s&tab=db_backup&action=%s&file=%s&_wpnonce=%s">Restore</a>', esc_attr( $_REQUEST['page'] ), 'restore', $item['title'], $restore_nonce ),
			'download' => sprintf( '<a href="?page=%s&tab=db_backup&action=%s&file=%s&_wpnonce=%s">Download</a>', esc_attr( $_REQUEST['page'] ), 'download', $item['title'], $download_nonce ),
			'delete' => sprintf( '<a href="?page=%s&tab=db_backup&action=%s&file=%s&_wpnonce=%s">Delete</a>', esc_attr( $_REQUEST['page'] ), 'delete', $item['title'], $delete_nonce )
		];
		
		return $item['date'] . $this->row_actions( $actions );
	}

	public function sort_data( $a, $b ) {
		
        // Set defaults
        $orderby = 'title';
        $order = 'asc';
        
        // If orderby is set, use this as the sort column
        if( ! empty( $_GET['orderby'] ) ) {
            $orderby = $_GET['orderby'];
        }
        
        // If order is set use this as the order
        if( ! empty( $_GET['order'] ) ) {
            $order = $_GET['order'];
        }
        
        $result = strcmp( $a[$orderby], $b[$orderby] );
        
        if( $order === 'asc' ) {
            return $result;
        }
        
        return -$result;
	}
	
	/**
	 * @description: Overwritten get_views function of WP_List_Table. Quick links for table data. (Import, Backup)
	 * @return: $views (array) => links' htmls
	 **/
	public function get_views(){
		$current = ( !empty( $_REQUEST['customvar'] ) ? $_REQUEST['customvar'] : 'all');
		
		//All link
		$class = ($current == 'all' ? ' class="current"' :'');
		$all_url = remove_query_arg('customvar');
		$views['all'] = "<a href='{$all_url }' {$class} >".__('All', 'weptile-wp-db-search-replace')." (".$this->counts['all'].")</a>";
		
		//Import link
		$import_url = add_query_arg('customvar', 'import');
		$class = ($current == 'import' ? ' class="current"' :'');
		$views['import'] = "<a href='{$import_url}' {$class} >".__('Import', 'weptile-wp-db-search-replace')." (".$this->counts['import'].")</a>";
		
		//Backup link
		$backup_url = add_query_arg('customvar', 'backup');
		$class = ($current == 'backup' ? ' class="current"' :'');
		$views['backup'] = "<a href='{$backup_url}' {$class} >".__('Backup', 'weptile-wp-db-search-replace')." (".$this->counts['backup'].")</a>";
		
		return $views;
	}
	
	public function column_cb( $item ) { // checkbox column on the left
		return sprintf(
			'<input type="checkbox" name="%1$s[]" value="%2$s" />',
			$this->_args['singular'],
			$item['title']
		);
	}
}