<?php
 
/**
 * Plugin Name: 		WP DB Search & Replace
 * Plugin URI: 			https://weptile.com
 * Description: 		Search & Replace any string or URL in the WordPress database. Search in tables, select tables and rows to replace for, convert http / https or www / non-www, backup database, import SQL or run a custom SQL query easily.
 * Version: 			0.2.0
 * Author:  			20script
 * Author URI: 			https://www.20script.ir
 * Text Domain: 		weptile-wp-db-search-replace
 * @package				WeptileWPDBSearchReplace
**/

defined( 'ABSPATH' ) or die;

if ( ! class_exists( 'Weptile_Backup_List' ) ) {
	require_once( ABSPATH . 'wp-content/plugins/weptile-wp-db-search-replace/weptile-backup-list.php');
}

class WeptileWPDBSR {
	
	/* Settings Tabs Object */
	private $tabs;
	
	private $upload_dir;
	
	private $pludin_dir_url;
	
	/* Weptile_Backup_List Object */
	private $backup_list;
	
	function __construct(){}

	function init(){
		add_action( 'admin_menu', 				array( $this, 'wpdbsr_add_submenu' ) );
		add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), array( $this, 'add_settings_link' ) );
		// ajax callbacks
		add_action( 'wp_ajax_search', 			array( $this, 'ajax_search_callback' ) );
		add_action( 'wp_ajax_replace', 			array( $this, 'ajax_replace_callback' ) );
		add_action( 'wp_ajax_httpHttps', 		array( $this, 'ajax_httpHttps_callback' ) );
		add_action( 'wp_ajax_wwwNoNwww', 		array( $this, 'ajax_wwwNoNwww_callback' ) );
		add_action( 'wp_ajax_backup', 			array( $this, 'ajax_backup_callback' ) );
		add_action( 'wp_ajax_restore', 			array( $this, 'ajax_restore_callback' ) );
		add_action( 'wp_ajax_sqlImport', 		array( $this, 'ajax_sqlImport_callback' ) );
		add_action( 'wp_ajax_customSqlImport', 	array( $this, 'ajax_customSqlImport_callback' ) );
		add_action( 'wp_ajax_tablePrefix', 		array( $this, 'ajax_tablePrefix_callback' ) );

		add_action(	'admin_init', 				array( $this, 'wpdbsr_set_variables' ) );
		add_action(	'admin_head', 				array( $this, 'wpdbsr_add_settings_notices' ) );
		add_action( 'admin_enqueue_scripts',	array( $this, 'wpdbsr_custom_scripts' ) );
	}
	
	/**
	 * @description: Set sub menu to tools.php for plugin settings when admin_menu action called.
	 **/
	function wpdbsr_add_submenu(){
	    add_submenu_page(
	    	'tools.php',
	    	__( 'Weptile WP DB Search & Replace Settings', 'weptile-wp-db-search-replace' ),
	    	__( 'DB Search & Replace', 'weptile-wp-db-search-replace' ),
	    	'manage_options',
	    	'wpdbsr_settings',
	    	array( $this, 'wpdbsr_settings_page_callback' ));
	}

	/**
	 * @description: Set Settings link under plugin title in plugins page when plugin_action_links_{plugin_basename} action called.
	 * @param: Settings links.
	 * @return: Updated links.
	 **/
	function add_settings_link( $links ) {
		array_unshift( $links, '<a href="' . esc_url( get_admin_url( null, 'tools.php?page=wpdbsr_settings' ) ) . '">Settings</a>' );
		return $links;
	}
	
	/**
	 * @description: Set variables when admin_init action called.
	 **/
	function wpdbsr_set_variables(){
		$this->upload_dir 		= wp_upload_dir();
		$this->pludin_dir_url 	= plugin_dir_url( __FILE__ );
		$this->backup_list 		= new Weptile_Backup_List();
		
		$this->tabs = array(
			'search_replace'	=> __('Search & Replace', 'weptile-wp-db-search-replace' ),
			'http_s' 			=> __('Http ⇌ Https', 'weptile-wp-db-search-replace' ),
			'www_non' 			=> __('www ⇌ non-www', 'weptile-wp-db-search-replace' ),
			'db_backup' 		=> __('Database Backup', 'weptile-wp-db-search-replace' ),
			'sql_import' 		=> __('SQL Import', 'weptile-wp-db-search-replace' ),
			'custom_sql_query' 	=> __('Custom SQL Query', 'weptile-wp-db-search-replace' ),
			'table_prefix' 		=> __('Change Table Prefix', 'weptile-wp-db-search-replace' ),
		);
	}
	
	/**
	 * @description: Set admin notice hooks when admin_head action called.
	 **/
	function wpdbsr_add_settings_notices() {
		if( isset( $_REQUEST['notice'] ) ){
			switch( $_REQUEST['notice'] ){
				case 'ds':
					add_action('admin_notices', array( $this, 'delete_success_notice' ) );
					break;
				case 'de':
					add_action('admin_notices', array( $this, 'delete_error_notice' ) );
					break;
				case 'rs':
					add_action('admin_notices', array( $this, 'restore_success_notice' ) );
					break;
				case 're':
					add_action('admin_notices', array( $this, 'restore_error_notice' ) );
					break;
				default: break;
			}
		}
	}
	
	/**
	 * @description: Register scripts when admin_enqueue_scripts action called.
	 **/
	function wpdbsr_custom_scripts(){
		wp_register_style( 'search_replace_css', $this->pludin_dir_url . 'css/search_replace.css', array(), '1.0.0' );
		wp_register_script( 'search_replace', $this->pludin_dir_url . 'js/search_replace.js', array('jquery'), '1.0.0', true  );
	}
	
	/**
	 * @description: Set page content callback for plugin submenu. Function called within wpdbsr_add_submenu.
	 **/
	function wpdbsr_settings_page_callback() {

		wp_enqueue_style( 'search_replace_css' );
		wp_enqueue_script( 'search_replace' );
		wp_localize_script( 'search_replace', 'search_replace_plugin', array(
			'ajaxURL' 						=> admin_url( 'admin-ajax.php' ),
			'tableText' 					=> __('Table', 'weptile-wp-db-search-replace'),
			'columnText' 					=> __('Column', 'weptile-wp-db-search-replace'),
			'contentText'					=> __('Content', 'weptile-wp-db-search-replace'),
			'replaceButtonText' 			=> __('Replace', 'weptile-wp-db-search-replace'),
			'selectAllText' 				=> __('Select All', 'weptile-wp-db-search-replace'),
			'allText' 						=> __('All', 'weptile-wp-db-search-replace'),
			'confirmTitle' 					=> __('Are you sure?', 'weptile-wp-db-search-replace'),
			'confirmText' 					=> __('Please make sure you have your backups before any crucial update. Click OK to proceed with the selected function.', 'weptile-wp-db-search-replace'),
			'errorTitle' 					=> __('Oops...', 'weptile-wp-db-search-replace'),
			'errorText' 					=> __('Something went wrong!', 'weptile-wp-db-search-replace'),
			'formatErrorText' 				=> __('Table prefix can only contain numbers and letters!', 'weptile-wp-db-search-replace'),
			'notFoundTitle' 				=> __('Not Found', 'weptile-wp-db-search-replace'),
			'emptyTitle' 					=> __('Choose an Option', 'weptile-wp-db-search-replace'),
			'successTitle' 					=> __('Done!', 'weptile-wp-db-search-replace'),
			'partiallyDone'					=> __('Partially Done!', 'weptile-wp-db-search-replace'),
			'replaceSuccessText' 			=> __(' Row(s) Replaced Successfuly!', 'weptile-wp-db-search-replace'),
			'replaceNoRowSelected' 			=> __('No row selected.', 'weptile-wp-db-search-replace'),
			'replaceTextEmpty' 				=> __('You need to enter a string to replace?', 'weptile-wp-db-search-replace'),
			'replaceLimitInfo' 				=> __(' Row(s) replaced successfuly from the total of ', 'weptile-wp-db-search-replace'),
			'replaceLimitInfo2' 			=> __('You may want to search and replace again for the remaining rows.', 'weptile-wp-db-search-replace'),
			'httpToHttpsSuccessText' 		=> __('All "http://YourSiteURL" occurences has been replaced with "https://YourSiteURL"!', 'weptile-wp-db-search-replace'),
			'httpsToHttpSuccessText' 		=> __('All "https://YourSiteURL" occurences has been replaced with "http://YourSiteURL"!', 'weptile-wp-db-search-replace'),
			'addWWWSuccessText' 			=> __('"www." has been added to all your site url occurences!', 'weptile-wp-db-search-replace'),
			'removeWWWSuccessText' 			=> __('"www." has been removed from all your site url occurences!', 'weptile-wp-db-search-replace'),
			'backupSuccessText' 			=> __('Backup Successful!', 'weptile-wp-db-search-replace'),
			'restoreSuccessText' 			=> __('Restore Successful!', 'weptile-wp-db-search-replace'),
			'restoreInfoText' 				=> __('You have to select a backup file and confirm.', 'weptile-wp-db-search-replace'),
			'sqlImportSuccessText' 			=> __('File Import Successful!', 'weptile-wp-db-search-replace'),
			'customSqlImportSuccessText'	=> __('Custom SQL Import Successful!', 'weptile-wp-db-search-replace'),
			'changeTablePrefixSuccessText' 	=> __('Table Prefix Changed Successfully!', 'weptile-wp-db-search-replace'),
			'searchResultEmpty' 			=> __('You should enter 3 characters at least.', 'weptile-wp-db-search-replace'),
			'noneSQLFileImportError' 		=> __('You can only import .sql files.', 'weptile-wp-db-search-replace'),
			));

		global $wpdb;
		$current_prefix = $wpdb->prefix;

		// detect current tab to decide which tab to show
		if( !empty( $_GET['tab'] ) ){
			$current_tab = $_GET['tab'];
		} else {
			$current_tab = 'search_replace';
		}

		?> 
		<div id="wp_db_search_replace" class="wrap">
			<h2>WP DB Search & Replace Settings</h2>
			<nav class="nav-tab-wrapper">
		<?php

		foreach($this->tabs as $tab => $tab_name) { // sub links
			?> 
			<a class="nav-tab <?php if($current_tab === $tab) { ?> nav-tab-active<?php } ?>" href="?page=<?php echo $_REQUEST['page']; ?>&tab=<?php echo $tab; ?>"><?php echo __($tab_name, 'weptile-wp-db-search-replace' ); ?> </a>
			<?php
		}

		?> </nav> <?php
		
		if( $current_tab === 'search_replace' ) {
			?> 	<div id="search_replace">
					<p class="search-settings-box">
						<input class="all-options sr-input" type="search" name="searchedText" />
						<input class="button" type="submit" name="searchButton" value="<?php echo __('Search', 'weptile-wp-db-search-replace' ); ?>" />
						<label class="search-setting-item"><input type="checkbox" name="caseSensitive"><strong class="ml-1"><?php echo __('Case Sensitive', 'weptile-wp-db-search-replace' ); ?></strong></label>
						<span class="spinner" style="float:none;"></span>
					</p>
					<div name="search_results_content">
					</div>
				</div>
			<?php
		}

		else if( $current_tab === 'http_s' ) {
			?> 	<div id="http_s">
					<h2><?php echo __('HTTP(S) Add/Remove', 'weptile-wp-db-search-replace' ); ?><span class="spinner" style="float:none;"></span></h2>
					<table class="form-table">
						<tbody>
							<tr>
								<th scope="row"><?php echo __('Http(s) Options', 'weptile-wp-db-search-replace' ); ?></th>
								<td>
									<fieldset>
										<label>
											<input type="radio" name="https" data-name="httpToHttps" value="" />
											<?php $url = $_SERVER['SERVER_NAME']; ?>
											<span><?php echo 'http://'.$url; ?>&nbsp;&nbsp;</span>
											<code>to <?php echo 'http<strong>s</strong>://'.$url; ?></code>
										</label> <br>
										<label>
											<input type="radio" name="https" data-name="httpsToHttp" value="" />
											<span><?php echo 'http<strong>s</strong>://'.$url; ?></span>
											<code>to <?php echo 'http://'.$url; ?></code>
										</label>
									</fieldset>
								</td>
							</tr>
						</tbody>
					</table>
					<input class="button-primary" type="submit" name="applyHttpsOptions" value="<?php echo __('Apply', 'weptile-wp-db-search-replace' ); ?>" />
				</div>
			<?php
		}

		else if( $current_tab === 'www_non' ) {
			?> 	<div id="www_non">
					<h2><?php echo __('WWW Add/Remove', 'weptile-wp-db-search-replace' ); ?><span class="spinner" style="float:none;"></span></h2>
					<table class="form-table">
						<tbody>
							<tr>
								<th scope="row"><?php echo __('WWW Options', 'weptile-wp-db-search-replace' ); ?></th>
								<td>
									<fieldset>
										<label>
											<input type="radio" name="wwwnon" data-name="addWWW" value="" />
											<?php $url = $_SERVER['SERVER_NAME']; ?>
											<span><?php echo $url; ?>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</span>
											<code>to <?php echo 'www.'.$url; ?></code>
										</label> <br>
										<label>
											<input type="radio" name="wwwnon" data-name="removeWWW" value="" />
											<span><?php echo 'www.'.$url; ?></span>
											<code>to <?php echo $url; ?></code>
										</label>
									</fieldset>
								</td>
							</tr>
						</tbody>
					</table>
					<input class="button-primary" type="submit" name="www_non_www" value="<?php echo __('Apply', 'weptile-wp-db-search-replace' ); ?>" />
				</div>
			<?php
		}

		else if( $current_tab === 'db_backup' ) {
			?> 	<div id="db_backup">
					<h2 class="db-title"><?php echo __('Database Backup', 'weptile-wp-db-search-replace' ); ?></h2>
					<input class="page-title-action" type="submit" name="backup" value="<?php echo __('Backup', 'weptile-wp-db-search-replace' ); ?>" /><span class="spinner" style="float:none;"></span>
					<div class="" name="restoreFiles">
						<form id="events-filter" method="get">
    						<input type="hidden" name="page" value="<?php echo $_REQUEST['page']; ?>" />
    						<input type="hidden" name="tab" value="<?php echo $_REQUEST['tab']; ?>" />
	                 		<?php 
								$this->backup_list->prepare_items();
								$this->backup_list->views();
								$this->backup_list->search_box('Search', 'search');
								$this->backup_list->display();
							?>
						</form>
                    </div>
				</div>
			<?php
		}

		else if( $current_tab === 'sql_import' ) {
			?> 	<div id="sql_import">
					<h2><?php echo __('SQL Import & Restore', 'weptile-wp-db-search-replace' ); ?><span class="spinner" style="float:none;"></span></h2>
					<div class="upload-plugin" style="display: block;">
						<p class="install-help"><?php echo __('If you have a database backup in .sql format, you may upload it here to replace the whole database.', 'weptile-wp-db-search-replace' ); ?></p>
						<div  class="wp-upload-form">
							<label class="screen-reader-text" for="sqlFileUpload"><?php echo __('Import SQL File (only)', 'weptile-wp-db-search-replace' ); ?></label>
							<input type="file" name="sqlFileUpload" id="sqlFileUpload" accept=".sql" />
							<input class="button" type="submit" name="sqlImport" value="<?php echo  __('Import and Execute', 'weptile-wp-db-search-replace' ); ?>" disabled="" />
						</div>
					</div>
				</div>
			<?php
		}

		else if( $current_tab === 'custom_sql_query' ) {
			?> 	<div id="custom_sql_query">
				<h2><?php echo __('Custom Query', 'weptile-wp-db-search-replace' ); ?><span class="spinner" style="float:none;"></span></h2>
					<table class="form-table">
						<tbody>
							<tr>
								<th scope="row"><?php echo __('SQL Command', 'weptile-wp-db-search-replace' ); ?></th>
								<td>
									<textarea rows="10" cols="80" name="customSqlText"></textarea>
								</td>
							</tr>
						</tbody>
					</table>
					<input class="button-primary" type="submit" name="customSqlImport" value="<?php echo __('Apply', 'weptile-wp-db-search-replace' ); ?>" />
					<div name="customSqlResults"></div>
				</div>
			<?php
		}

		else if( $current_tab === 'table_prefix' ) {
			?> 	<div id="table_prefix">
				<h2><?php echo __('WordPress Database Table Prefix Settings', 'weptile-wp-db-search-replace' ); ?><span class="spinner" style="float:none;"></span></h2>
					<table class="form-table">
						<tbody>
							<tr>
								<th scope="row"><?php echo __('Prefix', 'weptile-wp-db-search-replace' ); ?></th>
								<td>
									<input class="regular-text" type="text" name="tablePrefix" placeholder="<?php echo $current_prefix; ?>" />
									<br /><br />
									<span class="description">
										<?php echo __('Sample: ', 'weptile-wp-db-search-replace'); ?><code><strong><?php echo $current_prefix; ?></strong>options</code>
									</span>
								</td>
							</tr>
						</tbody>
					</table>
					<input class="button-primary" type="submit" name="changeTablePrefix" value="<?php echo __('Apply', 'weptile-wp-db-search-replace' ); ?>" />
				</div>
			<?php
		}

		?> </div> <?php
	}
	
	/**
	 * @description: Delete succesful admin notice.
	 **/
	public function delete_success_notice(){
		?>
		<div class="notice notice-success is-dismissible">
			<p><?php echo __('Files deleted succesfuly.', 'weptile-wp-db-search-replace'); ?></p>
		</div>
		<?php
	}
	
	/**
	 * @description: Delete error admin notice.
	 **/
	public function delete_error_notice(){
		?>
		<div class="notice notice-error is-dismissible">
			<p><?php echo __('No files deleted!', 'weptile-wp-db-search-replace'); ?></p>
		</div>
		<?php
	}
	
	/**
	 * @description: Restore succesful admin notice.
	 **/
	public function restore_success_notice(){
		?>
		<div class="notice notice-success is-dismissible">
			<p><?php echo __('Backup restored succesfully.', 'weptile-wp-db-search-replace'); ?></p>
		</div>
		<?php
	}
	
	/**
	 * @description: Restore error admin notice.
	 **/
	public function restore_error_notice(){
		?>
		<div class="notice notice-error is-dismissible">
			<p><?php echo __('Backup not restored!', 'weptile-wp-db-search-replace'); ?></p>
		</div>
		<?php
	}
	
	/**
	 * @description: Ajax callback for "search" action.
	 **/
	function ajax_search_callback(){
		
		/*
			$_POST['action'] = 'search',
			$_POST['text'] = 'searchedText'
			$_POST['isCaseSensitive'] = false
		*/
		$_POST['isCaseSensitive'] = filter_var( $_POST['isCaseSensitive'], FILTER_VALIDATE_BOOLEAN );
	  	wp_send_json( $this->search_on_db( $_POST ) );
	}
	
	/**
	 * @description: Ajax callback for "replace" action.
	 **/
	function ajax_replace_callback(){
		
		/**
			$_POST['action'] = 'replace',
			$_POST['oldText'] = 'oldText',
			$_POST['newText'] = 'newText',
			$_POST['isCaseSensitive'] = false,
			$_POST['updateRows'] = ['tableName', 'columnName', 'primary_key', 'key_value', 'isSerialized']
		*/
		$_POST['isCaseSensitive'] = filter_var( $_POST['isCaseSensitive'], FILTER_VALIDATE_BOOLEAN );
		wp_send_json( $this->replace_on_db( $_POST ) );
	}
	
	/**
	 * @description: Ajax callback for "httpHttps" action.
	 **/
	function ajax_httpHttps_callback(){
		wp_die( $this->replace_http_s_both( filter_var( $_POST['add_s'], FILTER_VALIDATE_BOOLEAN ) ) );
	}
	
	/**
	 * @description: Ajax callback for "wwwNoNwww" action.
	 **/
	function ajax_wwwNoNwww_callback(){
		wp_die( $this->add_remove_www( filter_var( $_POST['add_www'], FILTER_VALIDATE_BOOLEAN ) ) );
	}
	
	/**
	 * @description: Ajax callback for "backup" action.
	 **/
	function ajax_backup_callback(){
		
		$file_path = $this->upload_dir['basedir'] . '/weptile-wp-db-search-replace/backups/'. date("Y-m-d");
		
		$this->backup_list->backup_database('backup-' . time() . '.sql', $file_path);
		
		wp_die();
	}

	/**
	 * @description: Ajax callback for "sqlImport" action.
	 **/
	function ajax_sqlImport_callback(){
		wp_send_json( $this->sql_import( $_FILES['sqlFile'] ) );
	}

	/**
	 * @description: Ajax callback for "customSqlImport" action.
	 **/
	function ajax_customSqlImport_callback(){
		wp_send_json( $this->backup_list->custom_sql_import( $_POST['sql']) );
	}

	/**
	 * @description: Ajax callback for "tablePrefix" action.
	 **/
	function ajax_tablePrefix_callback(){
		$this->table_rename( $_POST['prefix']);
		wp_die( );
	}

	/**
	 * @description: Search given text on all tables and columns. Mark them unserialized or not.
	 * @param: [ {key} => {value} ]
	 * @return: [ 
	 * 				{tableName} => [ 
	 * 					{columnName} => [
	 * 						rows => [
	 * 							{primary_key_value} => [
	 * 								text => '',
	 * 								isSerialized => 1 or 0,
	 * 							],
	 * 							...
	 * 						]
	 * 					], 
	 * 					...,
	 * 				  	primary_key => '' 
	 * 				],
	 * 				...
	 * 			]
	 **/
	function search_on_db( $data ){
		
			global $wpdb;
	
			$tables = $wpdb->get_results( "SHOW TABLES" ); // get all table
	
			foreach($tables as $index => $value) {
				foreach($value as $tableName) {
	
					$columns = $wpdb->get_results( "DESCRIBE $tableName" ); // get all column for all tale
	
					$columnNames = array();
	
					foreach($columns as $column){
						$columnNames[] = $column->Field;
					}
	
					$tables_columns[$tableName] = $columnNames; // tableName => columnName array
				}
			}
	
			$found_rows = array();
	
			foreach( $tables_columns as $table => $columns ){
	
			 	if( $table == $wpdb->users ) // exclude user table from search
			 		continue;
	
				$primary_keys = $wpdb->get_row( "SHOW KEYS FROM $table WHERE Key_name = 'PRIMARY';" );
	
				if( ! $primary_keys )
					continue;
	
				foreach( $columns as $column ){
					// find the rows that includes searched text
					if( $data['isCaseSensitive'] ){ // case sensitive search
						$searched_text_results = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM $table WHERE $column LIKE BINARY %s OR $column LIKE BINARY %s", '%'.$wpdb->esc_like( $data['text'] ).'%', '%'.$wpdb->esc_like( str_replace( ' ', '%20', $data['text'] ) ).'%' ) );
					} else { // case insensitive search
						$searched_text_results = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM $table WHERE $column LIKE %s OR $column LIKE %s", '%'.$wpdb->esc_like( $data['text'] ).'%', '%'.$wpdb->esc_like( str_replace( ' ', '%20', $data['text'] ) ).'%' ) );
					}
					
					if( !empty( $searched_text_results ) ){	// if any result found		
		
						if( !isset( $found_rows[$table] ) )
							$found_rows[$table] = array(); // init a new array if that table is not initialized already
		
						if( !isset( $found_rows[$table][$column] ) )
							$found_rows[$table][$column] = array(); // init a new array if that column is not initialized already
		
						foreach( $searched_text_results as $index => $row ) {
							$unserialized_row = unserialize($row->$column); // check for each found row that is unserialized or not
		
							$found_rows[$table][$column]['rows'][$row->{$primary_keys->Column_name}]['text'] = $row->$column;
		
							if($unserialized_row){ // mark that row unserialized (1) or not (0)
								$found_rows[$table][$column]['rows'][$row->{$primary_keys->Column_name}]['isSerialized'] = 1;
							} else {
								$found_rows[$table][$column]['rows'][$row->{$primary_keys->Column_name}]['isSerialized'] = 0;
							}
		
						}
			
					}			
				}
		
				if( isset( $found_rows[$table] ) )
					$found_rows[$table]["primary_key"] = $primary_keys->Column_name; // set primary_key for each table (etc. id)
				
			}
			
			return $found_rows;
	}

	/**
	 * @description: Replace the given text with the searched text. Works for serialized arrays.
	 * @param: 	[
	 * 				{key} => {value},
	 * 				updateRows => [
	 * 					0 => ['tableName', 'columnName', 'primary_key', 'primary_key_value', 'isSerialized'],
	 * 					...
	 * 				]
	 * 			]
	 **/
	function replace_on_db( $data ){
	
		global $wpdb;
		
		if( ! is_array($data['updateRows'][0]) ){
			$updateRows = array();
			foreach($data['updateRows'] as $row){
				$updateRows[] = explode( '-', $row );
			}
			$data['updateRows'] = $updateRows;
		}
		
		foreach($data['updateRows'] as $index => $row){ // for each row that will replace with newText.
			if( count($row) < 5 || count(array_filter($row, function($el){ return $el === ''; })) > 0 ){ 
				// if somehow a row information is missing or empty break the loop. This is generally happen when http request size limit is not enough for all sent rows.
				break;
			}
			// find current row on database
			$result = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $row[0] WHERE $row[2]=%d ", $row[3] ) );
	
			$result = (array) $result; // array cast for object(stdClass)
				
			if($row[4] == 1){ // if row is serialized arrray
				
				$unserialized_array = unserialize( $result[$row[1]] ); // unserialize the column to search and replace the text
				
				if( $data['isCaseSensitive'] ){ // make string replace in json which is encoded from unserialized array
					$replaced_json = str_replace( $data['oldText'], $data['newText'], json_encode( $unserialized_array ) );
					$replaced_json = str_replace( str_replace( ' ', '%20', $data['oldText'] ), str_replace( ' ', '%20', $data['newText'] ), json_encode( $unserialized_array ) );
				} else {
					$replaced_json = str_ireplace( $data['oldText'], $data['newText'], json_encode( $unserialized_array ) );
					$replaced_json = str_ireplace( str_ireplace( ' ', '%20', $data['oldText'] ), str_ireplace( ' ', '%20', $data['newText'] ), json_encode( $unserialized_array ) );
				}
				
				$result[$row[1]] = serialize( json_decode( $replaced_json, true ) ); // decode json and serialize column again
				
			} else { // if row is not serialized array
				$escaped_result = json_encode( htmlspecialchars ( $result[$row[1]] ) ); // get row data and escape html
				
				if( $data['isCaseSensitive'] ){ // make replace on that escaped data
					$escaped_result = str_replace( $data['oldText'], $data['newText'], $escaped_result );
					$escaped_result = str_replace( str_replace( ' ', '%20', $data['oldText'] ), str_replace( ' ', '%20', $data['newText'] ), $escaped_result );
				} else {
					$escaped_result = str_ireplace( $data['oldText'], $data['newText'], $escaped_result );
					$escaped_result = str_ireplace( str_ireplace( ' ', '%20', $data['oldText'] ), str_ireplace( ' ', '%20', $data['newText'] ), $escaped_result );
				}
				
				$result[$row[1]] = htmlspecialchars_decode( json_decode( $escaped_result ) ); // decode that updated escaped data and replace with older
				
			}
				
			$wpdb->show_errors(); 
			$wpdb->query( $wpdb->prepare( "UPDATE $row[0] SET $row[1] = %s WHERE $row[2]=%d;", $result[$row[1]], $row[3] ) ); // make replacement on database
			$queries['query'][] = $wpdb->last_query; // get last sent query successful or fail
			$queries['error'][] = $wpdb->last_error; // get last error if does not exist it gets ''
		}
		$countQueries = count($queries['error']); // total number of queries
		$message = $countQueries - count(array_filter($queries['error'])) . '/' . $countQueries; // make a finished message for frontend like: (successful queries / all queries)
		return $message;
	}

	/**
	 * @description: Replace http to https or https to http in site url and it's occurences. This function includes two callback.
	 * @param: $add_s => true/false (if you want to add 's' to http, it means http to https, "$add_s=true", otherwise "$add_s=false")
	 * @return: true / false
	 **/
	function replace_http_s_both( $add_s ){
		
		$site_url = get_site_url();
	
		$pos_http = strpos($site_url, 'http:'); // check site url contains http
	
		$pos_https = strpos($site_url, 'https:'); // check site url contains https
	
		$data['isCaseSensitive'] = false;
	
		if( $add_s ) { // http to https
		
			if ($pos_http !== false) { // if site url starts with http
			
				$data['newText'] = str_replace( 'http:', 'https:', $site_url );
				$data['oldText'] = $site_url;
				
			} else if($pos_https !== false) { // if site url starts with https
			
				$data['newText'] = $site_url;
				$data['oldText'] = str_replace('https:', 'http:', $site_url);
				
			} else { // if site url does not include http or https
				return __('Site url does not include http or https.', 'weptile-wp-db-search-replace');
			}
	
		} else { // https to http
	
			if ($pos_http !== false) { // if site url starts with http
			
				$data['newText'] = $site_url;
				$data['oldText'] = str_replace('http:', 'https:', $site_url);
				
			} else if($pos_https !== false) { // if site url starts with https
			
				$data['newText'] = str_replace( 'https:', 'http:', $site_url );
				$data['oldText'] = $site_url;
				
			} else { // if site url does not include http or https
				return __('Site url does not include http or https.', 'weptile-wp-db-search-replace');
			}
	
		}
	
		$data = $this->fill_data_info_and_search( $data ); // get rows to update with new rows
		
		if( ! empty( $data['updateRows'] ) )
			$this->replace_on_db( $data ); // update rows
	
		if( strpos($data['oldText'], '://www.') !== false ){ // check replaced old text contains 'www'
			$data['oldText'] = str_replace('://www.', '://', $data['oldText']); // if replaced old text contains 'www' remove and update old text
		} else {
			$data['oldText'] = str_replace('://', '://www.', $data['oldText']); // if replaced old text not contains 'www' add update old text
		}
		
		$data['updateRows'] = array();
		
		$data = $this->fill_data_info_and_search( $data ); // try to find that updated old text on database
		
		if( ! empty( $data['updateRows'] ) )
			$this->replace_on_db( $data ); // if any occurrence exist replace them too
		
		 return true;
	}

	/**
	 * @description: This is a common part of replace_http_s_both and add_remove_www functions. It does fill the data['updateRows'] for replacement.
	 * @param: $data => array for the replace_on_db function.
	 * @return: new $data array with updated $data['updateRows'] array.
	 **/
	function fill_data_info_and_search( $data ) {
	
		$found_rows = $this->search_on_db( array('text' => $data['oldText'], 'isCaseSensitive' => $data['isCaseSensitive']) );
	
		if( empty( $found_rows ) ){ // if no row found return current data array without change
			return $data;
		}
	
		foreach($found_rows as $tableName => $table){
	
			$primary_key = $table['primary_key'];
	
			foreach($table as $columnName => $column) {
	
				if($columnName != 'primary_key'){
					foreach($column['rows'] as $key => $row){
						$data['updateRows'][] = [ $tableName, $columnName, $primary_key, $key, $row['isSerialized'] ]; // fill rows information
					}
				}
	
			}
		}
	
		return $data;
	}

	/**
	 * @description: Add or remove 'www' from site url and it's occurences. This function includes two callback. This works anyway if site url contains http(s) or not.
	 * @param: $add_www => to add 'www' send true, tor remove www send false.
	 * @return: true / false
	 **/
	function add_remove_www( $add_www ){
		
		$site_url = get_site_url();
		
		$data['isCaseSensitive'] = false;
		
		if( strpos($site_url, '://') !== false ){
			$pos = strpos($site_url, '://www.');
			
			if($add_www){ // add www to site url
		
				if ($pos !== false){ // www found in site url
		
					$data['oldText'] = str_replace('://www.', '://', $site_url); // make old text non-www
					$data['newText'] = $site_url; // site url has already www
		
				} else { // www not found in site url
		
					$data['oldText'] = $site_url; // site url does not have www
					$data['newText'] = str_replace('://', '://www.', $site_url); // make old text with www
		
				}
		
			} else { // remove www from site url
		
				if ($pos !== false){ // www found in site url
		
					$data['oldText'] = $site_url; // site url has already www
					$data['newText'] = str_replace('://www.', '://', $site_url); // make new text non-www
		
				} else { // www not found in site url
		
					$data['oldText'] = str_replace('://', '://www.', $site_url); // make old text with www
					$data['newText'] = $site_url; // site url does not have www
		
				}
		
			}
		} else {
			$hasWWW = (substr( $site_url, 0, 4 ) === 'www.');
			
			if($add_www){ // add www to site url
		
				if ( $hasWWW ){ // www found in site url
		
					$data['oldText'] = substr($site_url, 4); // make old text non-www
					$data['newText'] = $site_url; // site url has already www
		
				} else { // www not found in site url
		
					$data['oldText'] = $site_url; // site url does not have www
					$data['newText'] = 'www.' . $site_url; // make old text with www
		
				}
		
			} else { // remove www from site url
		
				if ( $hasWWW ){ // www found in site url
		
					$data['oldText'] = $site_url; // site url has already www
					$data['newText'] = substr($site_url, 4); // make new text non-www
		
				} else { // www not found in site url
		
					$data['oldText'] = 'www.' . $site_url; // make old text with www
					$data['newText'] = $site_url; // site url does not have www
		
				}
		
			}
		}
	
		$data = $this->fill_data_info_and_search( $data ); // try to find that old text on database and fill replaceable rows
	
		if( ! empty( $data['updateRows'] ) )
			$this->replace_on_db( $data ); // if any occurrence exist replace them too
	
		return true;
	}

	/**
	 * @description: Saves imported sql files inside imported-files folder under weptile-wp-db-search-replace.
	 * @param: $_FILES['file'] => sql file to import
	 **/
	function sql_import( $file ){
	
		$file_path = $this->upload_dir['basedir'] . '/weptile-wp-db-search-replace/broken-imported-files';
	
		if(! file_exists($file_path)) { // create folder if it does not exist.
			mkdir($file_path, 0777, true); // 0777 default mode for directories.
		}
	
		$info = pathinfo($file['name']);
	
		$ext = $info['extension']; // get the extension of the file
	
		$newname = $info['filename'].'-'. time() .'.'.$ext; // new file name with timestamp
	
		$target = $file_path.'/'.$newname; // target file path
	
		move_uploaded_file( $file['tmp_name'], $target); // move imported file to temp path till restore
	
		$errors = $this->backup_list->restore_file( $target ); // restore saved imported file
		
		if( count(array_filter($errors, function($el){ return $el !== ''; })) === 0 ) { // if any error exists after restoring imported file do not change file path otherwise move successful path
	
			$new_file_path = $this->upload_dir['basedir'] . '/weptile-wp-db-search-replace/backups/imported-files'; // path for successfuly imported files
	
			if(! file_exists($new_file_path)) { // create folder if it does not exist.
				mkdir($new_file_path, 0777, true); // 0777 default mode for directories.
			}
	
			rename( $target, $new_file_path.'/'.$newname);
		}
	
		return $errors;
	}

	/**
	 * @description: Change all tables' prefix's to given prefix.
	 * @param: $prefix (String) => given prefix
	 * @return: false => if error exists (echo error).
	 **/
	function table_rename( $prefix ){
		
		global $wpdb;
		
		$old_prefix = $wpdb->prefix; // get current prefix
		$prefix = wp_strip_all_tags( str_replace( ' ', '', str_replace( '_', '', $prefix) ) ) . '_'; // trim "_" and " " chars from entered prefix
	
		if($old_prefix === $prefix){ // if same prefix entered with the old one
			return __("You entered the same prefix as the current!", 'weptile-wp-db-search-replace');
		}
		
		$error = $wpdb->set_prefix( $prefix ); // set new prefix to wordpress db
		
		if ( is_wp_error( $error ) ){ // check new prefix is suitable for wordpress
			return __("Table prefix can only contain numbers and letters!", 'weptile-wp-db-search-replace');
		}
		// get temp backup
		$file_path = $this->upload_dir['basedir'] . '/weptile-wp-db-search-replace/tmp-backups'; // get temporary file path for temp backups
		
		$tempBackupName = 'before-' . $old_prefix . '-to-' . $prefix . '-' . time() . '.sql';
		
		$tempBackupPath = $file_path . '/' . $tempBackupName;
		
		$this->backup_list->backup_database($tempBackupName, $file_path); // get backup before changing prefix to restore any case of errors
		
		$tables = $wpdb->get_results( "SHOW TABLES", ARRAY_N ); // get all table
	
		$sql = "RENAME TABLE ";
		
		$dropSql = "DROP TABLE IF EXISTS ";
		
		foreach($tables as $table) {
				
			$newTableName = $prefix . explode('_', $table[0], 2)[1]; // make new table name with new prefix
			
			$sql .= "$table[0] TO $newTableName, "; // make rename sql
			
			$dropSql .= "$table[0], "; // make drop old table sql
		}
		
		$sql = rtrim($sql, ', ') . ';'; // remove last comma from sql command and finish
		
		$dropSql = rtrim($dropSql, ', ') . ';'; // remove last comma from sql command and finish
		
		if( $wpdb->query( $wpdb->prepare( $sql ) ) === false ){ // if anything goes wrong restore temp backup file and return error
			$this->backup_list->restore_file($tempBackupPath);
			return __('Table Rename Error!', 'weptile-wp-db-search-replace');
		}
		
		if( $wpdb->query( $wpdb->prepare( $dropSql ) ) === false ){ // if anything goes wrong restore temp backup file and return error
			$this->backup_list->restore_file($tempBackupPath);
			return __('Table Drop Error!', 'weptile-wp-db-search-replace');
		}
		
		$path = '../wp-config.php'; // get wp-config path to change prefix init
		
		$configFile = file( $path ); // get file
		
		foreach ( $configFile as $line_num => $line ) { // take each line of file
			switch ( substr( $line, 0, 15 ) ) {
				case '$table_prefix =': // if table prefix init found
					$configFile[$line_num] = str_replace( $old_prefix, $prefix, $line ); // replace old prefix init with new one
					break;
			}
		}
		
		@chmod( $path, 0777 );
		if ( is_writeable( $path ) ) { // if config file is writeable
			$handle = fopen( $path, 'w' );
			
			foreach ( $configFile as $line ) {
				fwrite( $handle, $line ); // write all lines back with updated prefix init line
			}
			
			fclose( $handle );
		} else { // if config file is not writeable restore temp backup file and return error
			$this->backup_list->restore_file($tempBackupPath);
			return __('wp_config.php is not writable!', 'weptile-wp-db-search-replace');
		}
		
		// start to replace prefix occurrences in _options table option_name column and _usermeta table meta_key column 
		
		$results = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM $prefix"."options WHERE option_name LIKE %s", '%'.$wpdb->esc_like( $old_prefix ).'%' ) ); // search _options table option_name column for old prefix
	
		$primary_key = $wpdb->get_row( "SHOW KEYS FROM $prefix"."options WHERE Key_name = 'PRIMARY';" );
		
		foreach($results as $row){
			if ( $wpdb->query( $wpdb->prepare( "UPDATE $prefix"."options SET option_name = REPLACE(option_name, '$old_prefix', '$prefix') WHERE $primary_key->Column_name=%d;", $row->{$primary_key->Column_name} ) ) === false ){ // update old prefix occurrences with new prefix in _options table option_name column if any error occurs restore temp backup file and return error
				$this->backup_list->restore_file($tempBackupPath);
				return __('Table: ' . $prefix . 'options, Column: option_name' .  $primary_key->Column_name . ' = ' . $row->{$primary_key->Column_name} . ' Update Error!', 'weptile-wp-db-search-replace');
			}
		}
		
		$results = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM $prefix"."usermeta WHERE meta_key LIKE %s", '%'.$wpdb->esc_like( $old_prefix ).'%' ) ); // search _usermeta table meta_key column for old prefix
		
		$primary_key = $wpdb->get_row( "SHOW KEYS FROM $prefix"."usermeta WHERE Key_name = 'PRIMARY';" );
		
		foreach($results as $row){
			if( $wpdb->query( $wpdb->prepare( "UPDATE $prefix"."usermeta SET meta_key = REPLACE(meta_key, '$old_prefix', '$prefix') WHERE $primary_key->Column_name=%d;", $row->{$primary_key->Column_name} ) ) === false ){ // update old prefix occurrences with new prefix in _usermeta table meta_key column if any error occurs restore temp backup file and return error
				$this->backup_list->restore_file($tempBackupPath);
				return __('Table: ' . $prefix . 'usermeta, Column: meta_key ' . $primary_key->Column_name . ' = ' . $row->{$primary_key->Column_name} . ' Update Error!', 'weptile-wp-db-search-replace');
			}
		}
		
		if(file_exists($tempBackupPath)){ // no error found and still temp backup file exists delete file from temporary folder
			unlink($tempBackupPath);
		}
	}

}

if ( class_exists( 'WeptileWPDBSR' ) ) {
	$wpdbsr = new WeptileWPDBSR();
	$wpdbsr->init();
}